import { useState, useRef, useEffect } from 'react';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
  function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
  return new (P || (P = Promise))(function (resolve, reject) {
      function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
      function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
      function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
      step((generator = generator.apply(thisArg, _arguments || [])).next());
  });
}

function __generator(thisArg, body) {
  var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
  return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
  function verb(n) { return function (v) { return step([n, v]); }; }
  function step(op) {
      if (f) throw new TypeError("Generator is already executing.");
      while (g && (g = 0, op[0] && (_ = 0)), _) try {
          if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
          if (y = 0, t) op = [op[0] & 2, t.value];
          switch (op[0]) {
              case 0: case 1: t = op; break;
              case 4: _.label++; return { value: op[1], done: false };
              case 5: _.label++; y = op[1]; op = [0]; continue;
              case 7: op = _.ops.pop(); _.trys.pop(); continue;
              default:
                  if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                  if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                  if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                  if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                  if (t[2]) _.ops.pop();
                  _.trys.pop(); continue;
          }
          op = body.call(thisArg, _);
      } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
      if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
  }
}

function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
      if (ar || !(i in from)) {
          if (!ar) ar = Array.prototype.slice.call(from, 0, i);
          ar[i] = from[i];
      }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}

var ALPHANUMERIC_REGEX = /^[a-z0-9]$/i;
var shouldPreventDefault = function (keyCode, isAlphaNumeric, isMeta) {
    if (isAlphaNumeric === void 0) { isAlphaNumeric = false; }
    if (isMeta === void 0) { isMeta = false; }
    var isAlphabet = keyCode >= 64 && keyCode <= 90;
    // Below flag also checks if the typeed key is from numpad
    var isNumeric = (keyCode >= 48 && keyCode <= 57) || (keyCode >= 96 && keyCode <= 105);
    //Crtl + v:
    if (isMeta && keyCode === 86) {
        return false;
    }
    // By default we only allow numbers to be pressed
    if (isNumeric)
        return false;
    // We only allow alphabets to be pressed when the isAplhaNumeric flag is true
    if (isAlphabet && isAlphaNumeric)
        return false;
    // Backspace
    if (keyCode === 8) {
        return false;
    }
    return true;
};
var getClipboardReadPermission = function () {
    return navigator.permissions.query({
        name: "clipboard-read",
    });
};
var getClipboardContent = function () {
    return navigator.clipboard.readText();
};
/**
 *
 * @param arr
 * @param currentFocusedIndex
 * This function will return the partially filled array when focused index is apart from 0.
 * The array before the focused index will be filled with existing values.
 */
var getFilledArray = function (arr, pastingArr, currentFocusedIndex) {
    var lastIndex = arr.length - 1;
    if (currentFocusedIndex > 0) {
        var remainingPlaces = lastIndex - currentFocusedIndex;
        var partialArray = pastingArr.slice(0, remainingPlaces + 1);
        return __spreadArray(__spreadArray([], arr.slice(0, currentFocusedIndex), true), partialArray, true);
    }
    else {
        // Starts pasting the values in the array from 0th index
        return __spreadArray(__spreadArray([], pastingArr, true), arr.slice(pastingArr.length - 1, lastIndex), true);
    }
};

var useOtp = function (props) {
    var arrayValue = props.arrayValue, _a = props.isAlphaNumeric, isAlphaNumeric = _a === void 0 ? false : _a;
    var _b = useState(arrayValue), array = _b[0], setArray = _b[1];
    var _c = useState(0), currentForcusedIndex = _c[0], setCurrentFocusedIndex = _c[1];
    var inputRefs = useRef([]);
    /**
     * A function that returns the necessary event handlers based on index.
     */
    var getEventHandlers = function (index) {
        var onChange = function (e) {
            // Change the arrayValue and update only when number key is pressed
            setArray(function (preValue) {
                var newArray = __spreadArray([], preValue, true);
                if (parseInt(e.target.value)) {
                    newArray[index] = parseInt(e.target.value);
                }
                else {
                    newArray[index] = e.target.value;
                }
                return newArray;
            });
        };
        var onFocus = function (e) {
            setCurrentFocusedIndex(index);
            e.target.focus();
        };
        var onKeyUp = function (e) {
            if (e.key === "Backspace") {
                if (index === 0) {
                    setCurrentFocusedIndex(0);
                }
                else {
                    setCurrentFocusedIndex(index - 1);
                    if (inputRefs &&
                        inputRefs.current &&
                        index === currentForcusedIndex) {
                        inputRefs.current[index - 1].focus();
                    }
                }
            }
            else {
                /**
                 * Update focus only when number key is pressed
                 * We do a -2 below because we don't want the last input to update the currentFocusedIndex
                 * If we allow it then we get array out of bound error.
                 * */
                if ((isAlphaNumeric ? ALPHANUMERIC_REGEX.test(e.key) : parseInt(e.key)) &&
                    index <= array.length - 2) {
                    setCurrentFocusedIndex(index + 1);
                    if (inputRefs &&
                        inputRefs.current &&
                        index === currentForcusedIndex) {
                        inputRefs.current[index + 1].focus();
                    }
                }
            }
        };
        // Preventing typing of any other keys except for 1 to 9 And backspace
        var onKeyDown = function (e) {
            if (shouldPreventDefault(e.which, isAlphaNumeric, e.metaKey)) {
                e.preventDefault();
            }
        };
        return {
            onKeyUp: onKeyUp,
            onKeyDown: onKeyDown,
            onFocus: onFocus,
            onChange: onChange,
        };
    };
    useEffect(function () {
        document.addEventListener("paste", function () { return __awaiter(void 0, void 0, void 0, function () {
            var copyPermission, clipboardContent, newArray, filledArray;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, getClipboardReadPermission()];
                    case 1:
                        copyPermission = _a.sent();
                        if (copyPermission.state === "denied") {
                            throw new Error("Not allowed to read clipboard.");
                        }
                        return [4 /*yield*/, getClipboardContent()];
                    case 2:
                        clipboardContent = _a.sent();
                        try {
                            newArray = clipboardContent.split("");
                            newArray = isAlphaNumeric
                                ? newArray
                                : newArray.map(function (num) { return Number(num); });
                            filledArray = getFilledArray(array, newArray, currentForcusedIndex);
                            setArray(filledArray);
                            // Below we update the current focused index and also focus to the last input
                            if (newArray.length < array.length && currentForcusedIndex === 0) {
                                setCurrentFocusedIndex(newArray.length - 1);
                                inputRefs.current[newArray.length - 1].focus();
                            }
                            else {
                                setCurrentFocusedIndex(array.length - 1);
                                inputRefs.current[array.length - 1].focus();
                            }
                        }
                        catch (err) {
                            console.error(err);
                        }
                        return [2 /*return*/];
                }
            });
        }); });
        return function () {
            window.removeEventListener("paste", function () {
                return console.log("Removed paste listner");
            });
        };
    }, [currentForcusedIndex, array, isAlphaNumeric]);
    var isComplete = array.every(function (value) { return value !== ""; });
    return {
        array: array,
        setArray: setArray,
        currentForcusedIndex: currentForcusedIndex,
        setCurrentFocusedIndex: setCurrentFocusedIndex,
        getEventHandlers: getEventHandlers,
        isComplete: isComplete,
        refs: inputRefs,
    };
};

export { useOtp };
//# sourceMappingURL=index.js.map
